%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Image spectrum estimation by using wavelet packet method, see paper     
%  ''2D Wavelet Packet Spectrum for Image Analysis'' by A. Atto, 
%                  Y. Berthoumieu and P. Bolon. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%             Natural images              %%%%%%%%%%%%%%%
ImgName={'testimages/D87.gif' 'testimages/D17.gif' 'testimages/Fabric.0009.png'...
    'testimages/Fabric.0011.png' 'testimages/D10.gif' 'testimages/D3.gif'};
Lin=length(ImgName);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Select and load input image
m=6; % Natural order in ImgName, 1<=m<=Lsn
IName=char(ImgName(m));
im = double(imread(IName));
im = double(im(:,:,1));

figure
imagesc(im);
colormap(gray)

%%
[lx, ly] = size(im);
nm = lx*ly;

%% Parameters for the wavelet packet decomposition
% Select a wavelet with large order (Examples: 'daubQ', 'symQ' for Q>8).
Wname='db7'; % a wavelet name, type 'help waveinfo' for details
J0=6;    % Decomposition level
if ( (nm/4^(J0+2)<1) || (nextpow2(min(lx,ly)/2^J0) <=1) )
    error('Error ---> Input image does not have enough samples: consider a smaller J0.');
end


%% Compute the image spectrum by using wavelet packet method
[imWPpsd,f1,f2] = WPspectrum2D(im,J0,Wname);
%sf = fft2(im); % sf est de dimension 512*512 ou 640*640 selon Vistex ou Brodatz


%%
%figure
figureFS = figure('Color',[1 1 1]);
axes3 = axes('Visible','off','Parent',figureFS,'CLim',[0 1]);
surfl(f1,f2,imWPpsd);
set(gcf,'Color','w')
hold on
stem3(f1,f2,imWPpsd,'k','MarkerSize',3);
view(axes3,[-225 20]);
title('WP spectrum - Full --> Support = [0,\pi]\times [0,\pi]')
%% ZOOM in the neighborhood of 0
if (J0>4)
    Ns1 = round(2^J0/2); % number of x-axis samples : 1:Ns1 ---> [0 pi/2]
    Ns2 = round(2^J0/2); % number of y-axis samples : 1:Ns2 ---> [0 pi/2]
    %%%% ZOOM ROTATE /
    %%
    figureZR = figure('Color',[1 1 1]);
    axes2 = axes('Visible','off','Parent',figureZR,'CLim',[0 1]);
    surfl(f1(1:Ns1,1:Ns2),f2(1:Ns1,1:Ns2),imWPpsd(1:Ns1,1:Ns2),'Parent',axes2);
    grid(axes2,'on');
    hold(axes2,'all');
    %set(gcf,'Color','w')
    view(axes2,[-225 20]);
    title('WP Spectrum over [0,\pi/2]\times [0,\pi/2]')
    %%
    %%%% ZOOM PROFILE /
    figureZP = figure('Color',[1 1 1]);
    axes1 = axes('Visible','off','Parent',figureZP,'CLim',[0 1]);
    surfl(f1(1:Ns1,1:Ns2),f2(1:Ns1,1:Ns2),imWPpsd(1:Ns1,1:Ns2),'Parent',axes1);
    grid(axes1,'on');
    hold(axes1,'all');
    %set(gcf,'Color','w')
    view(axes1,[-45-180 0]);
    title('WP Spectrum over [0,\pi/2]\times [0,\pi/2] - Profile view')
end
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright: Abdourrahmane.Atto@univ-savoie.fr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%